"""
This type stub file was generated by pyright.
"""

import datetime
import logging
from typing import Any, AsyncIterator, Callable, Coroutine, Generator, List, Literal, Optional, Sequence, TYPE_CHECKING, Tuple, Type, TypeVar, Union, overload
from .user import ClientUser, User
from .invite import Invite
from .template import Template
from .widget import Widget
from .guild import Guild, GuildChannel
from .emoji import Emoji
from .channel import DMChannel, GroupChannel, PartialMessageable
from .enums import ChannelType, Status
from .mentions import AllowedMentions
from .errors import *
from .flags import ApplicationFlags, Intents
from .gateway import *
from .activity import ActivityTypes, BaseActivity
from .voice_client import VoiceProtocol
from .webhook import Webhook
from .appinfo import AppInfo
from .ui.view import View
from .stage_instance import StageInstance
from .threads import Thread, ThreadMember
from .sticker import GuildSticker, StandardSticker, StickerPack
from types import TracebackType
from typing_extensions import Self
from .abc import Messageable, PrivateChannel, Snowflake, SnowflakeTime
from .app_commands import Command, ContextMenu
from .automod import AutoModAction, AutoModRule
from .ext.commands import AutoShardedBot, Bot, CommandError, Context
from .integrations import Integration
from .interactions import Interaction
from .member import Member, VoiceState
from .message import Message
from .raw_models import RawAppCommandPermissionsUpdateEvent, RawBulkMessageDeleteEvent, RawIntegrationDeleteEvent, RawMemberRemoveEvent, RawMessageDeleteEvent, RawMessageUpdateEvent, RawReactionActionEvent, RawReactionClearEmojiEvent, RawReactionClearEvent, RawThreadDeleteEvent, RawThreadMembersUpdate, RawThreadUpdateEvent, RawTypingEvent
from .reaction import Reaction
from .role import Role
from .scheduled_event import ScheduledEvent
from .audit_logs import AuditLogEntry

"""
The MIT License (MIT)

Copyright (c) 2015-present Rapptz

Permission is hereby granted, free of charge, to any person obtaining a
copy of this software and associated documentation files (the "Software"),
to deal in the Software without restriction, including without limitation
the rights to use, copy, modify, merge, publish, distribute, sublicense,
and/or sell copies of the Software, and to permit persons to whom the
Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
DEALINGS IN THE SOFTWARE.
"""
if TYPE_CHECKING:
    ...
__all__ = ('Client', )
T = TypeVar('T')
Coro = Coroutine[Any, Any, T]
CoroT = TypeVar('CoroT', bound=Callable[..., Coro[Any]])
_log = ...
class _LoopSentinel:
    __slots__ = ...
    def __getattr__(self, attr: str) -> None:
        ...
    


_loop: Any = ...
class Client:
    r"""Represents a client connection that connects to Discord.
    This class is used to interact with the Discord WebSocket and API.

    .. container:: operations

        .. describe:: async with x

            Asynchronously initialises the client and automatically cleans up.

            .. versionadded:: 2.0

    A number of options can be passed to the :class:`Client`.

    Parameters
    -----------
    max_messages: Optional[:class:`int`]
        The maximum number of messages to store in the internal message cache.
        This defaults to ``1000``. Passing in ``None`` disables the message cache.

        .. versionchanged:: 1.3
            Allow disabling the message cache and change the default size to ``1000``.
    proxy: Optional[:class:`str`]
        Proxy URL.
    proxy_auth: Optional[:class:`aiohttp.BasicAuth`]
        An object that represents proxy HTTP Basic Authorization.
    shard_id: Optional[:class:`int`]
        Integer starting at ``0`` and less than :attr:`.shard_count`.
    shard_count: Optional[:class:`int`]
        The total number of shards.
    application_id: :class:`int`
        The client's application ID.
    intents: :class:`Intents`
        The intents that you want to enable for the session. This is a way of
        disabling and enabling certain gateway events from triggering and being sent.

        .. versionadded:: 1.5

        .. versionchanged:: 2.0
            Parameter is now required.
    member_cache_flags: :class:`MemberCacheFlags`
        Allows for finer control over how the library caches members.
        If not given, defaults to cache as much as possible with the
        currently selected intents.

        .. versionadded:: 1.5
    chunk_guilds_at_startup: :class:`bool`
        Indicates if :func:`.on_ready` should be delayed to chunk all guilds
        at start-up if necessary. This operation is incredibly slow for large
        amounts of guilds. The default is ``True`` if :attr:`Intents.members`
        is ``True``.

        .. versionadded:: 1.5
    status: Optional[:class:`.Status`]
        A status to start your presence with upon logging on to Discord.
    activity: Optional[:class:`.BaseActivity`]
        An activity to start your presence with upon logging on to Discord.
    allowed_mentions: Optional[:class:`AllowedMentions`]
        Control how the client handles mentions by default on every message sent.

        .. versionadded:: 1.4
    heartbeat_timeout: :class:`float`
        The maximum numbers of seconds before timing out and restarting the
        WebSocket in the case of not receiving a HEARTBEAT_ACK. Useful if
        processing the initial packets take too long to the point of disconnecting
        you. The default timeout is 60 seconds.
    guild_ready_timeout: :class:`float`
        The maximum number of seconds to wait for the GUILD_CREATE stream to end before
        preparing the member cache and firing READY. The default timeout is 2 seconds.

        .. versionadded:: 1.4
    assume_unsync_clock: :class:`bool`
        Whether to assume the system clock is unsynced. This applies to the ratelimit handling
        code. If this is set to ``True``, the default, then the library uses the time to reset
        a rate limit bucket given by Discord. If this is ``False`` then your system clock is
        used to calculate how long to sleep for. If this is set to ``False`` it is recommended to
        sync your system clock to Google's NTP server.

        .. versionadded:: 1.3
    enable_debug_events: :class:`bool`
        Whether to enable events that are useful only for debugging gateway related information.

        Right now this involves :func:`on_socket_raw_receive` and :func:`on_socket_raw_send`. If
        this is ``False`` then those events will not be dispatched (due to performance considerations).
        To enable these events, this must be set to ``True``. Defaults to ``False``.

        .. versionadded:: 2.0
    http_trace: :class:`aiohttp.TraceConfig`
        The trace configuration to use for tracking HTTP requests the library does using ``aiohttp``.
        This allows you to check requests the library is using. For more information, check the
        `aiohttp documentation <https://docs.aiohttp.org/en/stable/client_advanced.html#client-tracing>`_.

        .. versionadded:: 2.0
    max_ratelimit_timeout: Optional[:class:`float`]
        The maximum number of seconds to wait when a non-global rate limit is encountered.
        If a request requires sleeping for more than the seconds passed in, then
        :exc:`~discord.RateLimited` will be raised. By default, there is no timeout limit.
        In order to prevent misuse and unnecessary bans, the minimum value this can be
        set to is ``30.0`` seconds.

        .. versionadded:: 2.0

    Attributes
    -----------
    ws
        The websocket gateway the client is currently connected to. Could be ``None``.
    """
    def __init__(self, *, intents: Intents, **options: Any) -> None:
        ...
    
    async def __aenter__(self) -> Self:
        ...
    
    async def __aexit__(self, exc_type: Optional[Type[BaseException]], exc_value: Optional[BaseException], traceback: Optional[TracebackType]) -> None:
        ...
    
    @property
    def latency(self) -> float:
        """:class:`float`: Measures latency between a HEARTBEAT and a HEARTBEAT_ACK in seconds.

        This could be referred to as the Discord WebSocket protocol latency.
        """
        ...
    
    def is_ws_ratelimited(self) -> bool:
        """:class:`bool`: Whether the websocket is currently rate limited.

        This can be useful to know when deciding whether you should query members
        using HTTP or via the gateway.

        .. versionadded:: 1.6
        """
        ...
    
    @property
    def user(self) -> Optional[ClientUser]:
        """Optional[:class:`.ClientUser`]: Represents the connected client. ``None`` if not logged in."""
        ...
    
    @property
    def guilds(self) -> Sequence[Guild]:
        """Sequence[:class:`.Guild`]: The guilds that the connected client is a member of."""
        ...
    
    @property
    def emojis(self) -> Sequence[Emoji]:
        """Sequence[:class:`.Emoji`]: The emojis that the connected client has."""
        ...
    
    @property
    def stickers(self) -> Sequence[GuildSticker]:
        """Sequence[:class:`.GuildSticker`]: The stickers that the connected client has.

        .. versionadded:: 2.0
        """
        ...
    
    @property
    def cached_messages(self) -> Sequence[Message]:
        """Sequence[:class:`.Message`]: Read-only list of messages the connected client has cached.

        .. versionadded:: 1.1
        """
        ...
    
    @property
    def private_channels(self) -> Sequence[PrivateChannel]:
        """Sequence[:class:`.abc.PrivateChannel`]: The private channels that the connected client is participating on.

        .. note::

            This returns only up to 128 most recent private channels due to an internal working
            on how Discord deals with private channels.
        """
        ...
    
    @property
    def voice_clients(self) -> List[VoiceProtocol]:
        """List[:class:`.VoiceProtocol`]: Represents a list of voice connections.

        These are usually :class:`.VoiceClient` instances.
        """
        ...
    
    @property
    def application_id(self) -> Optional[int]:
        """Optional[:class:`int`]: The client's application ID.

        If this is not passed via ``__init__`` then this is retrieved
        through the gateway when an event contains the data or after a call
        to :meth:`~discord.Client.login`. Usually after :func:`~discord.on_connect`
        is called.

        .. versionadded:: 2.0
        """
        ...
    
    @property
    def application_flags(self) -> ApplicationFlags:
        """:class:`~discord.ApplicationFlags`: The client's application flags.

        .. versionadded:: 2.0
        """
        ...
    
    @property
    def application(self) -> Optional[AppInfo]:
        """Optional[:class:`~discord.AppInfo`]: The client's application info.

        This is retrieved on :meth:`~discord.Client.login` and is not updated
        afterwards. This allows populating the application_id without requiring a
        gateway connection.

        This is ``None`` if accessed before :meth:`~discord.Client.login` is called.

        .. seealso:: The :meth:`~discord.Client.application_info` API call

        .. versionadded:: 2.0
        """
        ...
    
    def is_ready(self) -> bool:
        """:class:`bool`: Specifies if the client's internal cache is ready for use."""
        ...
    
    def dispatch(self, event: str, /, *args: Any, **kwargs: Any) -> None:
        ...
    
    async def on_error(self, event_method: str, /, *args: Any, **kwargs: Any) -> None:
        """|coro|

        The default error handler provided by the client.

        By default this logs to the library logger however it could be
        overridden to have a different implementation.
        Check :func:`~discord.on_error` for more details.

        .. versionchanged:: 2.0

            ``event_method`` parameter is now positional-only
            and instead of writing to ``sys.stderr`` it logs instead.
        """
        ...
    
    async def before_identify_hook(self, shard_id: Optional[int], *, initial: bool = ...) -> None:
        """|coro|

        A hook that is called before IDENTIFYing a session. This is useful
        if you wish to have more control over the synchronization of multiple
        IDENTIFYing clients.

        The default implementation sleeps for 5 seconds.

        .. versionadded:: 1.4

        Parameters
        ------------
        shard_id: :class:`int`
            The shard ID that requested being IDENTIFY'd
        initial: :class:`bool`
            Whether this IDENTIFY is the first initial IDENTIFY.
        """
        ...
    
    async def setup_hook(self) -> None:
        """|coro|

        A coroutine to be called to setup the bot, by default this is blank.

        To perform asynchronous setup after the bot is logged in but before
        it has connected to the Websocket, overwrite this coroutine.

        This is only called once, in :meth:`login`, and will be called before
        any events are dispatched, making it a better solution than doing such
        setup in the :func:`~discord.on_ready` event.

        .. warning::

            Since this is called *before* the websocket connection is made therefore
            anything that waits for the websocket will deadlock, this includes things
            like :meth:`wait_for` and :meth:`wait_until_ready`.

        .. versionadded:: 2.0
        """
        ...
    
    async def login(self, token: str) -> None:
        """|coro|

        Logs in the client with the specified credentials and
        calls the :meth:`setup_hook`.


        Parameters
        -----------
        token: :class:`str`
            The authentication token. Do not prefix this token with
            anything as the library will do it for you.

        Raises
        ------
        LoginFailure
            The wrong credentials are passed.
        HTTPException
            An unknown HTTP related error occurred,
            usually when it isn't 200 or the known incorrect credentials
            passing status code.
        """
        ...
    
    async def connect(self, *, reconnect: bool = ...) -> None:
        """|coro|

        Creates a websocket connection and lets the websocket listen
        to messages from Discord. This is a loop that runs the entire
        event system and miscellaneous aspects of the library. Control
        is not resumed until the WebSocket connection is terminated.

        Parameters
        -----------
        reconnect: :class:`bool`
            If we should attempt reconnecting, either due to internet
            failure or a specific failure on Discord's part. Certain
            disconnects that lead to bad state will not be handled (such as
            invalid sharding payloads or bad tokens).

        Raises
        -------
        GatewayNotFound
            If the gateway to connect to Discord is not found. Usually if this
            is thrown then there is a Discord API outage.
        ConnectionClosed
            The websocket connection has been terminated.
        """
        ...
    
    async def close(self) -> None:
        """|coro|

        Closes the connection to Discord.
        """
        ...
    
    def clear(self) -> None:
        """Clears the internal state of the bot.

        After this, the bot can be considered "re-opened", i.e. :meth:`is_closed`
        and :meth:`is_ready` both return ``False`` along with the bot's internal
        cache cleared.
        """
        ...
    
    async def start(self, token: str, *, reconnect: bool = ...) -> None:
        """|coro|

        A shorthand coroutine for :meth:`login` + :meth:`connect`.

        Parameters
        -----------
        token: :class:`str`
            The authentication token. Do not prefix this token with
            anything as the library will do it for you.
        reconnect: :class:`bool`
            If we should attempt reconnecting, either due to internet
            failure or a specific failure on Discord's part. Certain
            disconnects that lead to bad state will not be handled (such as
            invalid sharding payloads or bad tokens).

        Raises
        -------
        TypeError
            An unexpected keyword argument was received.
        """
        ...
    
    def run(self, token: str, *, reconnect: bool = ..., log_handler: Optional[logging.Handler] = ..., log_formatter: logging.Formatter = ..., log_level: int = ..., root_logger: bool = ...) -> None:
        """A blocking call that abstracts away the event loop
        initialisation from you.

        If you want more control over the event loop then this
        function should not be used. Use :meth:`start` coroutine
        or :meth:`connect` + :meth:`login`.

        This function also sets up the logging library to make it easier
        for beginners to know what is going on with the library. For more
        advanced users, this can be disabled by passing ``None`` to
        the ``log_handler`` parameter.

        .. warning::

            This function must be the last function to call due to the fact that it
            is blocking. That means that registration of events or anything being
            called after this function call will not execute until it returns.

        Parameters
        -----------
        token: :class:`str`
            The authentication token. Do not prefix this token with
            anything as the library will do it for you.
        reconnect: :class:`bool`
            If we should attempt reconnecting, either due to internet
            failure or a specific failure on Discord's part. Certain
            disconnects that lead to bad state will not be handled (such as
            invalid sharding payloads or bad tokens).
        log_handler: Optional[:class:`logging.Handler`]
            The log handler to use for the library's logger. If this is ``None``
            then the library will not set up anything logging related. Logging
            will still work if ``None`` is passed, though it is your responsibility
            to set it up.

            The default log handler if not provided is :class:`logging.StreamHandler`.

            .. versionadded:: 2.0
        log_formatter: :class:`logging.Formatter`
            The formatter to use with the given log handler. If not provided then it
            defaults to a colour based logging formatter (if available).

            .. versionadded:: 2.0
        log_level: :class:`int`
            The default log level for the library's logger. This is only applied if the
            ``log_handler`` parameter is not ``None``. Defaults to ``logging.INFO``.

            .. versionadded:: 2.0
        root_logger: :class:`bool`
            Whether to set up the root logger rather than the library logger.
            By default, only the library logger (``'discord'``) is set up. If this
            is set to ``True`` then the root logger is set up as well.

            Defaults to ``False``.

            .. versionadded:: 2.0
        """
        ...
    
    def is_closed(self) -> bool:
        """:class:`bool`: Indicates if the websocket connection is closed."""
        ...
    
    @property
    def activity(self) -> Optional[ActivityTypes]:
        """Optional[:class:`.BaseActivity`]: The activity being used upon
        logging in.
        """
        ...
    
    @activity.setter
    def activity(self, value: Optional[ActivityTypes]) -> None:
        ...
    
    @property
    def status(self) -> Status:
        """:class:`.Status`:
        The status being used upon logging on to Discord.

        .. versionadded: 2.0
        """
        ...
    
    @status.setter
    def status(self, value: Status) -> None:
        ...
    
    @property
    def allowed_mentions(self) -> Optional[AllowedMentions]:
        """Optional[:class:`~discord.AllowedMentions`]: The allowed mention configuration.

        .. versionadded:: 1.4
        """
        ...
    
    @allowed_mentions.setter
    def allowed_mentions(self, value: Optional[AllowedMentions]) -> None:
        ...
    
    @property
    def intents(self) -> Intents:
        """:class:`~discord.Intents`: The intents configured for this connection.

        .. versionadded:: 1.5
        """
        ...
    
    @property
    def users(self) -> List[User]:
        """List[:class:`~discord.User`]: Returns a list of all the users the bot can see."""
        ...
    
    def get_channel(self, id: int, /) -> Optional[Union[GuildChannel, Thread, PrivateChannel]]:
        """Returns a channel or thread with the given ID.

        .. versionchanged:: 2.0

            ``id`` parameter is now positional-only.

        Parameters
        -----------
        id: :class:`int`
            The ID to search for.

        Returns
        --------
        Optional[Union[:class:`.abc.GuildChannel`, :class:`.Thread`, :class:`.abc.PrivateChannel`]]
            The returned channel or ``None`` if not found.
        """
        ...
    
    def get_partial_messageable(self, id: int, *, guild_id: Optional[int] = ..., type: Optional[ChannelType] = ...) -> PartialMessageable:
        """Returns a partial messageable with the given channel ID.

        This is useful if you have a channel_id but don't want to do an API call
        to send messages to it.

        .. versionadded:: 2.0

        Parameters
        -----------
        id: :class:`int`
            The channel ID to create a partial messageable for.
        guild_id: Optional[:class:`int`]
            The optional guild ID to create a partial messageable for.

            This is not required to actually send messages, but it does allow the
            :meth:`~discord.PartialMessageable.jump_url` and
            :attr:`~discord.PartialMessageable.guild` properties to function properly.
        type: Optional[:class:`.ChannelType`]
            The underlying channel type for the partial messageable.

        Returns
        --------
        :class:`.PartialMessageable`
            The partial messageable
        """
        ...
    
    def get_stage_instance(self, id: int, /) -> Optional[StageInstance]:
        """Returns a stage instance with the given stage channel ID.

        .. versionadded:: 2.0

        Parameters
        -----------
        id: :class:`int`
            The ID to search for.

        Returns
        --------
        Optional[:class:`.StageInstance`]
            The stage instance or ``None`` if not found.
        """
        ...
    
    def get_guild(self, id: int, /) -> Optional[Guild]:
        """Returns a guild with the given ID.

        .. versionchanged:: 2.0

            ``id`` parameter is now positional-only.

        Parameters
        -----------
        id: :class:`int`
            The ID to search for.

        Returns
        --------
        Optional[:class:`.Guild`]
            The guild or ``None`` if not found.
        """
        ...
    
    def get_user(self, id: int, /) -> Optional[User]:
        """Returns a user with the given ID.

        .. versionchanged:: 2.0

            ``id`` parameter is now positional-only.

        Parameters
        -----------
        id: :class:`int`
            The ID to search for.

        Returns
        --------
        Optional[:class:`~discord.User`]
            The user or ``None`` if not found.
        """
        ...
    
    def get_emoji(self, id: int, /) -> Optional[Emoji]:
        """Returns an emoji with the given ID.

        .. versionchanged:: 2.0

            ``id`` parameter is now positional-only.

        Parameters
        -----------
        id: :class:`int`
            The ID to search for.

        Returns
        --------
        Optional[:class:`.Emoji`]
            The custom emoji or ``None`` if not found.
        """
        ...
    
    def get_sticker(self, id: int, /) -> Optional[GuildSticker]:
        """Returns a guild sticker with the given ID.

        .. versionadded:: 2.0

        .. note::

            To retrieve standard stickers, use :meth:`.fetch_sticker`.
            or :meth:`.fetch_premium_sticker_packs`.

        Returns
        --------
        Optional[:class:`.GuildSticker`]
            The sticker or ``None`` if not found.
        """
        ...
    
    def get_all_channels(self) -> Generator[GuildChannel, None, None]:
        """A generator that retrieves every :class:`.abc.GuildChannel` the client can 'access'.

        This is equivalent to: ::

            for guild in client.guilds:
                for channel in guild.channels:
                    yield channel

        .. note::

            Just because you receive a :class:`.abc.GuildChannel` does not mean that
            you can communicate in said channel. :meth:`.abc.GuildChannel.permissions_for` should
            be used for that.

        Yields
        ------
        :class:`.abc.GuildChannel`
            A channel the client can 'access'.
        """
        ...
    
    def get_all_members(self) -> Generator[Member, None, None]:
        """Returns a generator with every :class:`.Member` the client can see.

        This is equivalent to: ::

            for guild in client.guilds:
                for member in guild.members:
                    yield member

        Yields
        ------
        :class:`.Member`
            A member the client can see.
        """
        ...
    
    async def wait_until_ready(self) -> None:
        """|coro|

        Waits until the client's internal cache is all ready.

        .. warning::

            Calling this inside :meth:`setup_hook` can lead to a deadlock.
        """
        ...
    
    @overload
    async def wait_for(self, event: Literal['raw_app_command_permissions_update'], /, *, check: Optional[Callable[[RawAppCommandPermissionsUpdateEvent], bool]], timeout: Optional[float] = ...) -> RawAppCommandPermissionsUpdateEvent:
        ...
    
    @overload
    async def wait_for(self, event: Literal['app_command_completion'], /, *, check: Optional[Callable[[Interaction[Self], Union[Command[Any, ..., Any], ContextMenu]], bool]], timeout: Optional[float] = ...) -> Tuple[Interaction[Self], Union[Command[Any, ..., Any], ContextMenu]]:
        ...
    
    @overload
    async def wait_for(self, event: Literal['automod_rule_create', 'automod_rule_update', 'automod_rule_delete'], /, *, check: Optional[Callable[[AutoModRule], bool]], timeout: Optional[float] = ...) -> AutoModRule:
        ...
    
    @overload
    async def wait_for(self, event: Literal['automod_action'], /, *, check: Optional[Callable[[AutoModAction], bool]], timeout: Optional[float] = ...) -> AutoModAction:
        ...
    
    @overload
    async def wait_for(self, event: Literal['private_channel_update'], /, *, check: Optional[Callable[[GroupChannel, GroupChannel], bool]], timeout: Optional[float] = ...) -> Tuple[GroupChannel, GroupChannel]:
        ...
    
    @overload
    async def wait_for(self, event: Literal['private_channel_pins_update'], /, *, check: Optional[Callable[[PrivateChannel, datetime.datetime], bool]], timeout: Optional[float] = ...) -> Tuple[PrivateChannel, datetime.datetime]:
        ...
    
    @overload
    async def wait_for(self, event: Literal['guild_channel_delete', 'guild_channel_create'], /, *, check: Optional[Callable[[GuildChannel], bool]], timeout: Optional[float] = ...) -> GuildChannel:
        ...
    
    @overload
    async def wait_for(self, event: Literal['guild_channel_update'], /, *, check: Optional[Callable[[GuildChannel, GuildChannel], bool]], timeout: Optional[float] = ...) -> Tuple[GuildChannel, GuildChannel]:
        ...
    
    @overload
    async def wait_for(self, event: Literal['guild_channel_pins_update'], /, *, check: Optional[Callable[[Union[GuildChannel, Thread], Optional[datetime.datetime]], bool,]], timeout: Optional[float] = ...) -> Tuple[Union[GuildChannel, Thread], Optional[datetime.datetime]]:
        ...
    
    @overload
    async def wait_for(self, event: Literal['typing'], /, *, check: Optional[Callable[[Messageable, Union[User, Member], datetime.datetime], bool]], timeout: Optional[float] = ...) -> Tuple[Messageable, Union[User, Member], datetime.datetime]:
        ...
    
    @overload
    async def wait_for(self, event: Literal['raw_typing'], /, *, check: Optional[Callable[[RawTypingEvent], bool]], timeout: Optional[float] = ...) -> RawTypingEvent:
        ...
    
    @overload
    async def wait_for(self, event: Literal['connect', 'disconnect', 'ready', 'resumed'], /, *, check: Optional[Callable[[], bool]], timeout: Optional[float] = ...) -> None:
        ...
    
    @overload
    async def wait_for(self, event: Literal['shard_connect', 'shard_disconnect', 'shard_ready', 'shard_resumed'], /, *, check: Optional[Callable[[int], bool]], timeout: Optional[float] = ...) -> int:
        ...
    
    @overload
    async def wait_for(self, event: Literal['socket_event_type', 'socket_raw_receive'], /, *, check: Optional[Callable[[str], bool]], timeout: Optional[float] = ...) -> str:
        ...
    
    @overload
    async def wait_for(self, event: Literal['socket_raw_send'], /, *, check: Optional[Callable[[Union[str, bytes]], bool]], timeout: Optional[float] = ...) -> Union[str, bytes]:
        ...
    
    @overload
    async def wait_for(self, event: Literal['guild_available', 'guild_unavailable', 'guild_join', 'guild_remove',], /, *, check: Optional[Callable[[Guild], bool]], timeout: Optional[float] = ...) -> Guild:
        ...
    
    @overload
    async def wait_for(self, event: Literal['guild_update'], /, *, check: Optional[Callable[[Guild, Guild], bool]], timeout: Optional[float] = ...) -> Tuple[Guild, Guild]:
        ...
    
    @overload
    async def wait_for(self, event: Literal['guild_emojis_update'], /, *, check: Optional[Callable[[Guild, Sequence[Emoji], Sequence[Emoji]], bool]], timeout: Optional[float] = ...) -> Tuple[Guild, Sequence[Emoji], Sequence[Emoji]]:
        ...
    
    @overload
    async def wait_for(self, event: Literal['guild_stickers_update'], /, *, check: Optional[Callable[[Guild, Sequence[GuildSticker], Sequence[GuildSticker]], bool]], timeout: Optional[float] = ...) -> Tuple[Guild, Sequence[GuildSticker], Sequence[GuildSticker]]:
        ...
    
    @overload
    async def wait_for(self, event: Literal['invite_create', 'invite_delete'], /, *, check: Optional[Callable[[Invite], bool]], timeout: Optional[float] = ...) -> Invite:
        ...
    
    @overload
    async def wait_for(self, event: Literal['audit_log_entry_create'], /, *, check: Optional[Callable[[AuditLogEntry], bool]], timeout: Optional[float] = ...) -> AuditLogEntry:
        ...
    
    @overload
    async def wait_for(self, event: Literal['integration_create', 'integration_update'], /, *, check: Optional[Callable[[Integration], bool]], timeout: Optional[float] = ...) -> Integration:
        ...
    
    @overload
    async def wait_for(self, event: Literal['guild_integrations_update'], /, *, check: Optional[Callable[[Guild], bool]], timeout: Optional[float] = ...) -> Guild:
        ...
    
    @overload
    async def wait_for(self, event: Literal['webhooks_update'], /, *, check: Optional[Callable[[GuildChannel], bool]], timeout: Optional[float] = ...) -> GuildChannel:
        ...
    
    @overload
    async def wait_for(self, event: Literal['raw_integration_delete'], /, *, check: Optional[Callable[[RawIntegrationDeleteEvent], bool]], timeout: Optional[float] = ...) -> RawIntegrationDeleteEvent:
        ...
    
    @overload
    async def wait_for(self, event: Literal['interaction'], /, *, check: Optional[Callable[[Interaction[Self]], bool]], timeout: Optional[float] = ...) -> Interaction[Self]:
        ...
    
    @overload
    async def wait_for(self, event: Literal['member_join', 'member_remove'], /, *, check: Optional[Callable[[Member], bool]], timeout: Optional[float] = ...) -> Member:
        ...
    
    @overload
    async def wait_for(self, event: Literal['raw_member_remove'], /, *, check: Optional[Callable[[RawMemberRemoveEvent], bool]], timeout: Optional[float] = ...) -> RawMemberRemoveEvent:
        ...
    
    @overload
    async def wait_for(self, event: Literal['member_update', 'presence_update'], /, *, check: Optional[Callable[[Member, Member], bool]], timeout: Optional[float] = ...) -> Tuple[Member, Member]:
        ...
    
    @overload
    async def wait_for(self, event: Literal['user_update'], /, *, check: Optional[Callable[[User, User], bool]], timeout: Optional[float] = ...) -> Tuple[User, User]:
        ...
    
    @overload
    async def wait_for(self, event: Literal['member_ban'], /, *, check: Optional[Callable[[Guild, Union[User, Member]], bool]], timeout: Optional[float] = ...) -> Tuple[Guild, Union[User, Member]]:
        ...
    
    @overload
    async def wait_for(self, event: Literal['member_unban'], /, *, check: Optional[Callable[[Guild, User], bool]], timeout: Optional[float] = ...) -> Tuple[Guild, User]:
        ...
    
    @overload
    async def wait_for(self, event: Literal['message', 'message_delete'], /, *, check: Optional[Callable[[Message], bool]], timeout: Optional[float] = ...) -> Message:
        ...
    
    @overload
    async def wait_for(self, event: Literal['message_edit'], /, *, check: Optional[Callable[[Message, Message], bool]], timeout: Optional[float] = ...) -> Tuple[Message, Message]:
        ...
    
    @overload
    async def wait_for(self, event: Literal['bulk_message_delete'], /, *, check: Optional[Callable[[List[Message]], bool]], timeout: Optional[float] = ...) -> List[Message]:
        ...
    
    @overload
    async def wait_for(self, event: Literal['raw_message_edit'], /, *, check: Optional[Callable[[RawMessageUpdateEvent], bool]], timeout: Optional[float] = ...) -> RawMessageUpdateEvent:
        ...
    
    @overload
    async def wait_for(self, event: Literal['raw_message_delete'], /, *, check: Optional[Callable[[RawMessageDeleteEvent], bool]], timeout: Optional[float] = ...) -> RawMessageDeleteEvent:
        ...
    
    @overload
    async def wait_for(self, event: Literal['raw_bulk_message_delete'], /, *, check: Optional[Callable[[RawBulkMessageDeleteEvent], bool]], timeout: Optional[float] = ...) -> RawBulkMessageDeleteEvent:
        ...
    
    @overload
    async def wait_for(self, event: Literal['reaction_add', 'reaction_remove'], /, *, check: Optional[Callable[[Reaction, Union[Member, User]], bool]], timeout: Optional[float] = ...) -> Tuple[Reaction, Union[Member, User]]:
        ...
    
    @overload
    async def wait_for(self, event: Literal['reaction_clear'], /, *, check: Optional[Callable[[Message, List[Reaction]], bool]], timeout: Optional[float] = ...) -> Tuple[Message, List[Reaction]]:
        ...
    
    @overload
    async def wait_for(self, event: Literal['reaction_clear_emoji'], /, *, check: Optional[Callable[[Reaction], bool]], timeout: Optional[float] = ...) -> Reaction:
        ...
    
    @overload
    async def wait_for(self, event: Literal['raw_reaction_add', 'raw_reaction_remove'], /, *, check: Optional[Callable[[RawReactionActionEvent], bool]], timeout: Optional[float] = ...) -> RawReactionActionEvent:
        ...
    
    @overload
    async def wait_for(self, event: Literal['raw_reaction_clear'], /, *, check: Optional[Callable[[RawReactionClearEvent], bool]], timeout: Optional[float] = ...) -> RawReactionClearEvent:
        ...
    
    @overload
    async def wait_for(self, event: Literal['raw_reaction_clear_emoji'], /, *, check: Optional[Callable[[RawReactionClearEmojiEvent], bool]], timeout: Optional[float] = ...) -> RawReactionClearEmojiEvent:
        ...
    
    @overload
    async def wait_for(self, event: Literal['guild_role_create', 'guild_role_delete'], /, *, check: Optional[Callable[[Role], bool]], timeout: Optional[float] = ...) -> Role:
        ...
    
    @overload
    async def wait_for(self, event: Literal['guild_role_update'], /, *, check: Optional[Callable[[Role, Role], bool]], timeout: Optional[float] = ...) -> Tuple[Role, Role]:
        ...
    
    @overload
    async def wait_for(self, event: Literal['scheduled_event_create', 'scheduled_event_delete'], /, *, check: Optional[Callable[[ScheduledEvent], bool]], timeout: Optional[float] = ...) -> ScheduledEvent:
        ...
    
    @overload
    async def wait_for(self, event: Literal['scheduled_event_user_add', 'scheduled_event_user_remove'], /, *, check: Optional[Callable[[ScheduledEvent, User], bool]], timeout: Optional[float] = ...) -> Tuple[ScheduledEvent, User]:
        ...
    
    @overload
    async def wait_for(self, event: Literal['stage_instance_create', 'stage_instance_delete'], /, *, check: Optional[Callable[[StageInstance], bool]], timeout: Optional[float] = ...) -> StageInstance:
        ...
    
    @overload
    async def wait_for(self, event: Literal['stage_instance_update'], /, *, check: Optional[Callable[[StageInstance, StageInstance], bool]], timeout: Optional[float] = ...) -> Coroutine[Any, Any, Tuple[StageInstance, StageInstance]]:
        ...
    
    @overload
    async def wait_for(self, event: Literal['thread_create', 'thread_join', 'thread_remove', 'thread_delete'], /, *, check: Optional[Callable[[Thread], bool]], timeout: Optional[float] = ...) -> Thread:
        ...
    
    @overload
    async def wait_for(self, event: Literal['thread_update'], /, *, check: Optional[Callable[[Thread, Thread], bool]], timeout: Optional[float] = ...) -> Tuple[Thread, Thread]:
        ...
    
    @overload
    async def wait_for(self, event: Literal['raw_thread_update'], /, *, check: Optional[Callable[[RawThreadUpdateEvent], bool]], timeout: Optional[float] = ...) -> RawThreadUpdateEvent:
        ...
    
    @overload
    async def wait_for(self, event: Literal['raw_thread_delete'], /, *, check: Optional[Callable[[RawThreadDeleteEvent], bool]], timeout: Optional[float] = ...) -> RawThreadDeleteEvent:
        ...
    
    @overload
    async def wait_for(self, event: Literal['thread_member_join', 'thread_member_remove'], /, *, check: Optional[Callable[[ThreadMember], bool]], timeout: Optional[float] = ...) -> ThreadMember:
        ...
    
    @overload
    async def wait_for(self, event: Literal['raw_thread_member_remove'], /, *, check: Optional[Callable[[RawThreadMembersUpdate], bool]], timeout: Optional[float] = ...) -> RawThreadMembersUpdate:
        ...
    
    @overload
    async def wait_for(self, event: Literal['voice_state_update'], /, *, check: Optional[Callable[[Member, VoiceState, VoiceState], bool]], timeout: Optional[float] = ...) -> Tuple[Member, VoiceState, VoiceState]:
        ...
    
    @overload
    async def wait_for(self: Union[Bot, AutoShardedBot], event: Literal["command", "command_completion"], /, *, check: Optional[Callable[[Context[Any]], bool]] = ..., timeout: Optional[float] = ...) -> Context[Any]:
        ...
    
    @overload
    async def wait_for(self: Union[Bot, AutoShardedBot], event: Literal["command_error"], /, *, check: Optional[Callable[[Context[Any], CommandError], bool]] = ..., timeout: Optional[float] = ...) -> Tuple[Context[Any], CommandError]:
        ...
    
    @overload
    async def wait_for(self, event: str, /, *, check: Optional[Callable[..., bool]] = ..., timeout: Optional[float] = ...) -> Any:
        ...
    
    def wait_for(self, event: str, /, *, check: Optional[Callable[..., bool]] = ..., timeout: Optional[float] = ...) -> Coro[Any]:
        """|coro|

        Waits for a WebSocket event to be dispatched.

        This could be used to wait for a user to reply to a message,
        or to react to a message, or to edit a message in a self-contained
        way.

        The ``timeout`` parameter is passed onto :func:`asyncio.wait_for`. By default,
        it does not timeout. Note that this does propagate the
        :exc:`asyncio.TimeoutError` for you in case of timeout and is provided for
        ease of use.

        In case the event returns multiple arguments, a :class:`tuple` containing those
        arguments is returned instead. Please check the
        :ref:`documentation <discord-api-events>` for a list of events and their
        parameters.

        This function returns the **first event that meets the requirements**.

        Examples
        ---------

        Waiting for a user reply: ::

            @client.event
            async def on_message(message):
                if message.content.startswith('$greet'):
                    channel = message.channel
                    await channel.send('Say hello!')

                    def check(m):
                        return m.content == 'hello' and m.channel == channel

                    msg = await client.wait_for('message', check=check)
                    await channel.send(f'Hello {msg.author}!')

        Waiting for a thumbs up reaction from the message author: ::

            @client.event
            async def on_message(message):
                if message.content.startswith('$thumb'):
                    channel = message.channel
                    await channel.send('Send me that \N{THUMBS UP SIGN} reaction, mate')

                    def check(reaction, user):
                        return user == message.author and str(reaction.emoji) == '\N{THUMBS UP SIGN}'

                    try:
                        reaction, user = await client.wait_for('reaction_add', timeout=60.0, check=check)
                    except asyncio.TimeoutError:
                        await channel.send('\N{THUMBS DOWN SIGN}')
                    else:
                        await channel.send('\N{THUMBS UP SIGN}')

        .. versionchanged:: 2.0

            ``event`` parameter is now positional-only.


        Parameters
        ------------
        event: :class:`str`
            The event name, similar to the :ref:`event reference <discord-api-events>`,
            but without the ``on_`` prefix, to wait for.
        check: Optional[Callable[..., :class:`bool`]]
            A predicate to check what to wait for. The arguments must meet the
            parameters of the event being waited for.
        timeout: Optional[:class:`float`]
            The number of seconds to wait before timing out and raising
            :exc:`asyncio.TimeoutError`.

        Raises
        -------
        asyncio.TimeoutError
            If a timeout is provided and it was reached.

        Returns
        --------
        Any
            Returns no arguments, a single argument, or a :class:`tuple` of multiple
            arguments that mirrors the parameters passed in the
            :ref:`event reference <discord-api-events>`.
        """
        ...
    
    def event(self, coro: CoroT, /) -> CoroT:
        """A decorator that registers an event to listen to.

        You can find more info about the events on the :ref:`documentation below <discord-api-events>`.

        The events must be a :ref:`coroutine <coroutine>`, if not, :exc:`TypeError` is raised.

        Example
        ---------

        .. code-block:: python3

            @client.event
            async def on_ready():
                print('Ready!')

        .. versionchanged:: 2.0

            ``coro`` parameter is now positional-only.

        Raises
        --------
        TypeError
            The coroutine passed is not actually a coroutine.
        """
        ...
    
    async def change_presence(self, *, activity: Optional[BaseActivity] = ..., status: Optional[Status] = ...) -> None:
        """|coro|

        Changes the client's presence.

        Example
        ---------

        .. code-block:: python3

            game = discord.Game("with the API")
            await client.change_presence(status=discord.Status.idle, activity=game)

        .. versionchanged:: 2.0
            Removed the ``afk`` keyword-only parameter.

        .. versionchanged:: 2.0
            This function will now raise :exc:`TypeError` instead of
            ``InvalidArgument``.

        Parameters
        ----------
        activity: Optional[:class:`.BaseActivity`]
            The activity being done. ``None`` if no currently active activity is done.
        status: Optional[:class:`.Status`]
            Indicates what status to change to. If ``None``, then
            :attr:`.Status.online` is used.

        Raises
        ------
        TypeError
            If the ``activity`` parameter is not the proper type.
        """
        ...
    
    async def fetch_guilds(self, *, limit: Optional[int] = ..., before: Optional[SnowflakeTime] = ..., after: Optional[SnowflakeTime] = ..., with_counts: bool = ...) -> AsyncIterator[Guild]:
        """Retrieves an :term:`asynchronous iterator` that enables receiving your guilds.

        .. note::

            Using this, you will only receive :attr:`.Guild.owner`, :attr:`.Guild.icon`,
            :attr:`.Guild.id`, :attr:`.Guild.name`, :attr:`.Guild.approximate_member_count`,
            and :attr:`.Guild.approximate_presence_count` per :class:`.Guild`.

        .. note::

            This method is an API call. For general usage, consider :attr:`guilds` instead.

        Examples
        ---------

        Usage ::

            async for guild in client.fetch_guilds(limit=150):
                print(guild.name)

        Flattening into a list ::

            guilds = [guild async for guild in client.fetch_guilds(limit=150)]
            # guilds is now a list of Guild...

        All parameters are optional.

        Parameters
        -----------
        limit: Optional[:class:`int`]
            The number of guilds to retrieve.
            If ``None``, it retrieves every guild you have access to. Note, however,
            that this would make it a slow operation.
            Defaults to ``200``.

            .. versionchanged:: 2.0

                The default has been changed to 200.

        before: Union[:class:`.abc.Snowflake`, :class:`datetime.datetime`]
            Retrieves guilds before this date or object.
            If a datetime is provided, it is recommended to use a UTC aware datetime.
            If the datetime is naive, it is assumed to be local time.
        after: Union[:class:`.abc.Snowflake`, :class:`datetime.datetime`]
            Retrieve guilds after this date or object.
            If a datetime is provided, it is recommended to use a UTC aware datetime.
            If the datetime is naive, it is assumed to be local time.
        with_counts: :class:`bool`
            Whether to include count information in the guilds. This fills the
            :attr:`.Guild.approximate_member_count` and :attr:`.Guild.approximate_presence_count`
            attributes without needing any privileged intents. Defaults to ``True``.

            .. versionadded:: 2.3

        Raises
        ------
        HTTPException
            Getting the guilds failed.

        Yields
        --------
        :class:`.Guild`
            The guild with the guild data parsed.
        """
        ...
    
    async def fetch_template(self, code: Union[Template, str]) -> Template:
        """|coro|

        Gets a :class:`.Template` from a discord.new URL or code.

        Parameters
        -----------
        code: Union[:class:`.Template`, :class:`str`]
            The Discord Template Code or URL (must be a discord.new URL).

        Raises
        -------
        NotFound
            The template is invalid.
        HTTPException
            Getting the template failed.

        Returns
        --------
        :class:`.Template`
            The template from the URL/code.
        """
        ...
    
    async def fetch_guild(self, guild_id: int, /, *, with_counts: bool = ...) -> Guild:
        """|coro|

        Retrieves a :class:`.Guild` from an ID.

        .. note::

            Using this, you will **not** receive :attr:`.Guild.channels`, :attr:`.Guild.members`,
            :attr:`.Member.activity` and :attr:`.Member.voice` per :class:`.Member`.

        .. note::

            This method is an API call. For general usage, consider :meth:`get_guild` instead.

        .. versionchanged:: 2.0

            ``guild_id`` parameter is now positional-only.


        Parameters
        -----------
        guild_id: :class:`int`
            The guild's ID to fetch from.
        with_counts: :class:`bool`
            Whether to include count information in the guild. This fills the
            :attr:`.Guild.approximate_member_count` and :attr:`.Guild.approximate_presence_count`
            attributes without needing any privileged intents. Defaults to ``True``.

            .. versionadded:: 2.0

        Raises
        ------
        Forbidden
            You do not have access to the guild.
        HTTPException
            Getting the guild failed.

        Returns
        --------
        :class:`.Guild`
            The guild from the ID.
        """
        ...
    
    async def create_guild(self, *, name: str, icon: bytes = ..., code: str = ...) -> Guild:
        """|coro|

        Creates a :class:`.Guild`.

        Bot accounts in more than 10 guilds are not allowed to create guilds.

        .. versionchanged:: 2.0
            ``name`` and ``icon`` parameters are now keyword-only. The ``region`` parameter has been removed.

        .. versionchanged:: 2.0
            This function will now raise :exc:`ValueError` instead of
            ``InvalidArgument``.

        Parameters
        ----------
        name: :class:`str`
            The name of the guild.
        icon: Optional[:class:`bytes`]
            The :term:`py:bytes-like object` representing the icon. See :meth:`.ClientUser.edit`
            for more details on what is expected.
        code: :class:`str`
            The code for a template to create the guild with.

            .. versionadded:: 1.4

        Raises
        ------
        HTTPException
            Guild creation failed.
        ValueError
            Invalid icon image format given. Must be PNG or JPG.

        Returns
        -------
        :class:`.Guild`
            The guild created. This is not the same guild that is
            added to cache.
        """
        ...
    
    async def fetch_stage_instance(self, channel_id: int, /) -> StageInstance:
        """|coro|

        Gets a :class:`.StageInstance` for a stage channel id.

        .. versionadded:: 2.0

        Parameters
        -----------
        channel_id: :class:`int`
            The stage channel ID.

        Raises
        -------
        NotFound
            The stage instance or channel could not be found.
        HTTPException
            Getting the stage instance failed.

        Returns
        --------
        :class:`.StageInstance`
            The stage instance from the stage channel ID.
        """
        ...
    
    async def fetch_invite(self, url: Union[Invite, str], *, with_counts: bool = ..., with_expiration: bool = ..., scheduled_event_id: Optional[int] = ...) -> Invite:
        """|coro|

        Gets an :class:`.Invite` from a discord.gg URL or ID.

        .. note::

            If the invite is for a guild you have not joined, the guild and channel
            attributes of the returned :class:`.Invite` will be :class:`.PartialInviteGuild` and
            :class:`.PartialInviteChannel` respectively.

        Parameters
        -----------
        url: Union[:class:`.Invite`, :class:`str`]
            The Discord invite ID or URL (must be a discord.gg URL).
        with_counts: :class:`bool`
            Whether to include count information in the invite. This fills the
            :attr:`.Invite.approximate_member_count` and :attr:`.Invite.approximate_presence_count`
            fields.
        with_expiration: :class:`bool`
            Whether to include the expiration date of the invite. This fills the
            :attr:`.Invite.expires_at` field.

            .. versionadded:: 2.0
        scheduled_event_id: Optional[:class:`int`]
            The ID of the scheduled event this invite is for.

            .. note::

                It is not possible to provide a url that contains an ``event_id`` parameter
                when using this parameter.

            .. versionadded:: 2.0

        Raises
        -------
        ValueError
            The url contains an ``event_id``, but ``scheduled_event_id`` has also been provided.
        NotFound
            The invite has expired or is invalid.
        HTTPException
            Getting the invite failed.

        Returns
        --------
        :class:`.Invite`
            The invite from the URL/ID.
        """
        ...
    
    async def delete_invite(self, invite: Union[Invite, str], /) -> None:
        """|coro|

        Revokes an :class:`.Invite`, URL, or ID to an invite.

        You must have :attr:`~.Permissions.manage_channels` in
        the associated guild to do this.

        .. versionchanged:: 2.0

            ``invite`` parameter is now positional-only.

        Parameters
        ----------
        invite: Union[:class:`.Invite`, :class:`str`]
            The invite to revoke.

        Raises
        -------
        Forbidden
            You do not have permissions to revoke invites.
        NotFound
            The invite is invalid or expired.
        HTTPException
            Revoking the invite failed.
        """
        ...
    
    async def fetch_widget(self, guild_id: int, /) -> Widget:
        """|coro|

        Gets a :class:`.Widget` from a guild ID.

        .. note::

            The guild must have the widget enabled to get this information.

        .. versionchanged:: 2.0

            ``guild_id`` parameter is now positional-only.

        Parameters
        -----------
        guild_id: :class:`int`
            The ID of the guild.

        Raises
        -------
        Forbidden
            The widget for this guild is disabled.
        HTTPException
            Retrieving the widget failed.

        Returns
        --------
        :class:`.Widget`
            The guild's widget.
        """
        ...
    
    async def application_info(self) -> AppInfo:
        """|coro|

        Retrieves the bot's application information.

        Raises
        -------
        HTTPException
            Retrieving the information failed somehow.

        Returns
        --------
        :class:`.AppInfo`
            The bot's application information.
        """
        ...
    
    async def fetch_user(self, user_id: int, /) -> User:
        """|coro|

        Retrieves a :class:`~discord.User` based on their ID.
        You do not have to share any guilds with the user to get this information,
        however many operations do require that you do.

        .. note::

            This method is an API call. If you have :attr:`discord.Intents.members` and member cache enabled, consider :meth:`get_user` instead.

        .. versionchanged:: 2.0

            ``user_id`` parameter is now positional-only.

        Parameters
        -----------
        user_id: :class:`int`
            The user's ID to fetch from.

        Raises
        -------
        NotFound
            A user with this ID does not exist.
        HTTPException
            Fetching the user failed.

        Returns
        --------
        :class:`~discord.User`
            The user you requested.
        """
        ...
    
    async def fetch_channel(self, channel_id: int, /) -> Union[GuildChannel, PrivateChannel, Thread]:
        """|coro|

        Retrieves a :class:`.abc.GuildChannel`, :class:`.abc.PrivateChannel`, or :class:`.Thread` with the specified ID.

        .. note::

            This method is an API call. For general usage, consider :meth:`get_channel` instead.

        .. versionadded:: 1.2

        .. versionchanged:: 2.0

            ``channel_id`` parameter is now positional-only.

        Raises
        -------
        InvalidData
            An unknown channel type was received from Discord.
        HTTPException
            Retrieving the channel failed.
        NotFound
            Invalid Channel ID.
        Forbidden
            You do not have permission to fetch this channel.

        Returns
        --------
        Union[:class:`.abc.GuildChannel`, :class:`.abc.PrivateChannel`, :class:`.Thread`]
            The channel from the ID.
        """
        ...
    
    async def fetch_webhook(self, webhook_id: int, /) -> Webhook:
        """|coro|

        Retrieves a :class:`.Webhook` with the specified ID.

        .. versionchanged:: 2.0

            ``webhook_id`` parameter is now positional-only.

        Raises
        --------
        HTTPException
            Retrieving the webhook failed.
        NotFound
            Invalid webhook ID.
        Forbidden
            You do not have permission to fetch this webhook.

        Returns
        ---------
        :class:`.Webhook`
            The webhook you requested.
        """
        ...
    
    async def fetch_sticker(self, sticker_id: int, /) -> Union[StandardSticker, GuildSticker]:
        """|coro|

        Retrieves a :class:`.Sticker` with the specified ID.

        .. versionadded:: 2.0

        Raises
        --------
        HTTPException
            Retrieving the sticker failed.
        NotFound
            Invalid sticker ID.

        Returns
        --------
        Union[:class:`.StandardSticker`, :class:`.GuildSticker`]
            The sticker you requested.
        """
        ...
    
    async def fetch_premium_sticker_packs(self) -> List[StickerPack]:
        """|coro|

        Retrieves all available premium sticker packs.

        .. versionadded:: 2.0

        Raises
        -------
        HTTPException
            Retrieving the sticker packs failed.

        Returns
        ---------
        List[:class:`.StickerPack`]
            All available premium sticker packs.
        """
        ...
    
    async def create_dm(self, user: Snowflake) -> DMChannel:
        """|coro|

        Creates a :class:`.DMChannel` with this user.

        This should be rarely called, as this is done transparently for most
        people.

        .. versionadded:: 2.0

        Parameters
        -----------
        user: :class:`~discord.abc.Snowflake`
            The user to create a DM with.

        Returns
        -------
        :class:`.DMChannel`
            The channel that was created.
        """
        ...
    
    def add_view(self, view: View, *, message_id: Optional[int] = ...) -> None:
        """Registers a :class:`~discord.ui.View` for persistent listening.

        This method should be used for when a view is comprised of components
        that last longer than the lifecycle of the program.

        .. versionadded:: 2.0

        Parameters
        ------------
        view: :class:`discord.ui.View`
            The view to register for dispatching.
        message_id: Optional[:class:`int`]
            The message ID that the view is attached to. This is currently used to
            refresh the view's state during message update events. If not given
            then message update events are not propagated for the view.

        Raises
        -------
        TypeError
            A view was not passed.
        ValueError
            The view is not persistent or is already finished. A persistent view has no timeout
            and all their components have an explicitly provided custom_id.
        """
        ...
    
    @property
    def persistent_views(self) -> Sequence[View]:
        """Sequence[:class:`.View`]: A sequence of persistent views added to the client.

        .. versionadded:: 2.0
        """
        ...
    


