"""
This type stub file was generated by pyright.
"""

import ctypes
from typing import Any, List, Literal, Optional, TYPE_CHECKING, Tuple, TypeVar, TypedDict, overload
from .errors import DiscordException

"""
The MIT License (MIT)

Copyright (c) 2015-present Rapptz

Permission is hereby granted, free of charge, to any person obtaining a
copy of this software and associated documentation files (the "Software"),
to deal in the Software without restriction, including without limitation
the rights to use, copy, modify, merge, publish, distribute, sublicense,
and/or sell copies of the Software, and to permit persons to whom the
Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
DEALINGS IN THE SOFTWARE.
"""
if TYPE_CHECKING:
    T = TypeVar('T')
    BAND_CTL = Literal['narrow', 'medium', 'wide', 'superwide', 'full']
    SIGNAL_CTL = Literal['auto', 'voice', 'music']
class BandCtl(TypedDict):
    narrow: int
    medium: int
    wide: int
    superwide: int
    full: int
    ...


class SignalCtl(TypedDict):
    auto: int
    voice: int
    music: int
    ...


__all__ = ('Encoder', 'OpusError', 'OpusNotLoaded')
_log = ...
c_int_ptr = ...
c_int16_ptr = ...
c_float_ptr = ...
_lib: Any = ...
class EncoderStruct(ctypes.Structure):
    ...


class DecoderStruct(ctypes.Structure):
    ...


EncoderStructPtr = ...
DecoderStructPtr = ...
OK = ...
BAD_ARG = ...
APPLICATION_AUDIO = ...
APPLICATION_VOIP = ...
APPLICATION_LOWDELAY = ...
CTL_SET_BITRATE = ...
CTL_SET_BANDWIDTH = ...
CTL_SET_FEC = ...
CTL_SET_PLP = ...
CTL_SET_SIGNAL = ...
CTL_SET_GAIN = ...
CTL_LAST_PACKET_DURATION = ...
band_ctl: BandCtl = ...
signal_ctl: SignalCtl = ...
exported_functions: List[Tuple[Any, ...]] = ...
def libopus_loader(name: str) -> Any:
    ...

def load_opus(name: str) -> None:
    """Loads the libopus shared library for use with voice.

    If this function is not called then the library uses the function
    :func:`ctypes.util.find_library` and then loads that one if available.

    Not loading a library and attempting to use PCM based AudioSources will
    lead to voice not working.

    This function propagates the exceptions thrown.

    .. warning::

        The bitness of the library must match the bitness of your python
        interpreter. If the library is 64-bit then your python interpreter
        must be 64-bit as well. Usually if there's a mismatch in bitness then
        the load will throw an exception.

    .. note::

        On Windows, this function should not need to be called as the binaries
        are automatically loaded.

    .. note::

        On Windows, the .dll extension is not necessary. However, on Linux
        the full extension is required to load the library, e.g. ``libopus.so.1``.
        On Linux however, :func:`ctypes.util.find_library` will usually find the library automatically
        without you having to call this.

    Parameters
    ----------
    name: :class:`str`
        The filename of the shared library.
    """
    ...

def is_loaded() -> bool:
    """Function to check if opus lib is successfully loaded either
    via the :func:`ctypes.util.find_library` call of :func:`load_opus`.

    This must return ``True`` for voice to work.

    Returns
    -------
    :class:`bool`
        Indicates if the opus library has been loaded.
    """
    ...

class OpusError(DiscordException):
    """An exception that is thrown for libopus related errors.

    Attributes
    ----------
    code: :class:`int`
        The error code returned.
    """
    def __init__(self, code: int) -> None:
        ...
    


class OpusNotLoaded(DiscordException):
    """An exception that is thrown for when libopus is not loaded."""
    ...


class _OpusStruct:
    SAMPLING_RATE = ...
    CHANNELS = ...
    FRAME_LENGTH = ...
    SAMPLE_SIZE = ...
    SAMPLES_PER_FRAME = ...
    FRAME_SIZE = ...
    @staticmethod
    def get_opus_version() -> str:
        ...
    


class Encoder(_OpusStruct):
    def __init__(self, application: int = ...) -> None:
        ...
    
    def __del__(self) -> None:
        ...
    
    def set_bitrate(self, kbps: int) -> int:
        ...
    
    def set_bandwidth(self, req: BAND_CTL) -> None:
        ...
    
    def set_signal_type(self, req: SIGNAL_CTL) -> None:
        ...
    
    def set_fec(self, enabled: bool = ...) -> None:
        ...
    
    def set_expected_packet_loss_percent(self, percentage: float) -> None:
        ...
    
    def encode(self, pcm: bytes, frame_size: int) -> bytes:
        ...
    


class Decoder(_OpusStruct):
    def __init__(self) -> None:
        ...
    
    def __del__(self) -> None:
        ...
    
    @staticmethod
    def packet_get_nb_frames(data: bytes) -> int:
        """Gets the number of frames in an Opus packet"""
        ...
    
    @staticmethod
    def packet_get_nb_channels(data: bytes) -> int:
        """Gets the number of channels in an Opus packet"""
        ...
    
    @classmethod
    def packet_get_samples_per_frame(cls, data: bytes) -> int:
        """Gets the number of samples per frame from an Opus packet"""
        ...
    
    def set_gain(self, dB: float) -> int:
        """Sets the decoder gain in dB, from -128 to 128."""
        ...
    
    def set_volume(self, mult: float) -> int:
        """Sets the output volume as a float percent, i.e. 0.5 for 50%, 1.75 for 175%, etc."""
        ...
    
    @overload
    def decode(self, data: bytes, *, fec: bool) -> bytes:
        ...
    
    @overload
    def decode(self, data: Literal[None], *, fec: Literal[False]) -> bytes:
        ...
    
    def decode(self, data: Optional[bytes], *, fec: bool = ...) -> bytes:
        ...
    


